<?php

declare(strict_types=1);

namespace Rowlinson\Api;

use DateTimeImmutable;
use Rowlinson\Api\Collections\PaginatedCollection;
use Rowlinson\Api\Requests\Order\DeliveryLocation;
use Rowlinson\Api\Requests\Order\Item;
use Rowlinson\Api\Requests\Order\PlaceOrderRequest;
use Rowlinson\Api\Requests\Order\UpdateOrderTypeRequest;
use Rowlinson\Api\Responses\Order\OrderResponse;
use Rowlinson\Api\Responses\Order\OrdersResponse;
use Rowlinson\Api\Responses\Order\OrderTypeResponse;

class OrderClient extends AbstractClient
{
    /**
     * Place a new order with the supplied details, ignoring the current basket
     *
     * @param array<Item> $items
     */
    public function placeOrder(
        array $items,
        ?string $deliveryLocationId,
        ?DeliveryLocation $newDeliveryLocation,
        ?string $organisation,
        ?string $recipientName,
        ?string $deliveryInstructions,
        ?string $customerReference,
        ?string $sourceName,
        ?string $orderType
    ): void {
        $this->post(
            'orders/place',
            new PlaceOrderRequest(
                $items,
                $organisation,
                $deliveryLocationId,
                $newDeliveryLocation,
                $deliveryInstructions,
                $customerReference,
                $recipientName,
                $sourceName,
                $orderType
            )
        );
    }

    /**
     * Get the orders for the organisation of the currently logged in user
     *
     * @return PaginatedCollection<OrdersResponse>
     */
    public function getOrders(int $page = 1): PaginatedCollection
    {
        $response = $this->get('orders?' . http_build_query([
            'page' => $page,
        ]));

        $orderResponses = $this->serializer->deserialize(
            (string)$response->getBody(),
            'array<' . OrdersResponse::class . '>', // @phpstan-ignore-line
            'json'
        );

        return new PaginatedCollection(
            $orderResponses,
            $page,
            $response->getPaginationPerPage(),
            $response->getPaginationTotalItems()
        );
    }

    /**
     * Get the details of an individual order
     */
    public function getOrder(string $orderNumber): ?OrderResponse
    {
        $response = $this->get('orders/' . $orderNumber);

        return $this->serializer->deserialize((string)$response->getBody(), OrderResponse::class, 'json');
    }

    /**
     * Get the available forward order types for the currently authenticated user
     *
     * @param bool $all Whether to retrieve all order types, only available for admin users
     * @return PaginatedCollection<OrderTypeResponse>
     */
    public function getOrderTypes(bool $all = false): PaginatedCollection
    {
        $qs = ($all) ? '?' . http_build_query([ 'all' => 1 ]) : '';
        $response = $this->get('orders/types' . $qs);

        /** @var array<OrderTypeResponse> $orderTypeResponses */
        $orderTypeResponses = $this->serializer->deserialize(
            (string)$response->getBody(),
            'array<' . OrderTypeResponse::class . '>', // @phpstan-ignore-line
            'json'
        );

        return new PaginatedCollection(
            $orderTypeResponses,
            1,
            count($orderTypeResponses),
            count($orderTypeResponses)
        );
    }

    public function updateOrderType(
        string $alias,
        ?DateTimeImmutable $dueDate,
        ?bool $visible,
        ?DateTimeImmutable $cutOffTime,
        ?int $workingDays
    ): void {
        $this->put('orders/types/' . $alias, new UpdateOrderTypeRequest($dueDate, $visible, $cutOffTime, $workingDays));
    }
}
